<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:file="java:java.io.File">
	<xsl:output method="html"/>
	<!-- <xsl:strip-space elements="*"/> -->
	<xsl:variable name="comments" select="count(//contentModel[name(./preceding-sibling::*[1])='comment'])"/>

	<xsl:param name="SEPERATOR_PIPE_LABEL">choice</xsl:param> <!-- "OR - Group" -->
	<xsl:param name="SEPERATOR_COMMA_LABEL">sequence</xsl:param> <!-- "AND - Group" -->

	<xsl:param name="OCCURENCE_PLUS_LABEL">(one or more)</xsl:param> 
	<xsl:param name="OCCURENCE_ASTERISK_LABEL">(zero or more)</xsl:param> 
	<xsl:param name="OCCURENCE_QUESTION_LABEL">(optional)</xsl:param> 

	<xsl:variable name="CONST_ALPHABET">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>
	<xsl:key name="elements" match="//contentModel" use="substring(@ename, 1, 1)" />

	<xsl:variable name="filename" select="file:getName(file:new(string(/dtd/@sysid)))"/>

	<xsl:variable name="root_elements" select="/dtd/externalSubset/contentModel[not(@ename=//element/@name)]"/>
	<xsl:variable name="count_root_elements" select="count($root_elements)"/>

	<xsl:template match="dtd">
		<html>
			<head>
				<title>HTML documentation of "<xsl:value-of select="$filename"/>"</title>
				<style type="text/css">
					<xsl:comment>
						a:link { text-decoration:none; font-weight:normal; }
						a:visited { text-decoration:none; font-weight:normal; }
						a:hover { text-decoration:underline; font-weight:normal; }
						a:active { text-decoration:none; font-weight:normal; }
						a:focus { text-decoration:underline; font-weight:normal; }
						a { white-space:nowrap; }

						dl { margin-left:20px; margin-top:0px; margin-bottom:0px; }
						table { margin-top:-2px; margin-bottom:-2px; }
						pre {margin-top:-1px; margin-bottom:-1px;}
						body { font-family:arial,helvetica,sans-serif }
						td { font-family:arial,helvetica,sans-serif; }

					<!--	ul {list-style-type:disc }
						ul ul {list-style-type:circle }
						ul ul ul {list-style-type:square }
						ul ul ul ul {list-style-type:disc }
						ul ul ul ul ul {list-style-type:circle }
						ul ul ul ul ul ul {list-style-type:square }
						ul ul ul ul ul ul ul {list-style-type:disc }
						ul ul ul ul ul ul ul ul {list-style-type:circle }
						ul ul ul ul ul ul ul ul ul {list-style-type:square }
						ul ul ul ul ul ul ul ul ul ul {list-style-type:disc }
						ul ul ul ul ul ul ul ul ul ul ul {list-style-type:circle}
						ul ul ul ul ul ul ul ul ul ul ul ul {list-style-type:square }-->
					</xsl:comment>
				</style>
			</head>
			<body>
				<a name="top"></a>
				<h2>automatically generated HTML documentation of "<i><xsl:value-of select="$filename"/></i>"</h2>
				<xsl:apply-templates select="/dtd/externalSubset/contentModel[1]/preceding-sibling::comment[($comments=1) or (name(./following-sibling::*[1])='comment')]" mode="comment"/>

				<br/>
				<xsl:choose>
					<xsl:when test="$count_root_elements=0">
						<b>Root Element: </b> <i>could not determine</i>
 					</xsl:when>
					<xsl:when test="$count_root_elements=1">
						<b>Root Element: </b> <a href="#{@ename}"><xsl:value-of select="$root_elements/@ename"/></a> 
					</xsl:when>
					<xsl:otherwise>
						<b>possible Root Element(s): </b>
						<xsl:for-each select="$root_elements">
							<xsl:sort select="@ename"/>
							<a href="#{@ename}"><xsl:value-of select="@ename"/></a>
							<xsl:if test="position() != last()">, </xsl:if>
						</xsl:for-each>
					</xsl:otherwise>
				</xsl:choose>

				<p/><big><b><xsl:value-of select="count(/dtd/externalSubset/contentModel)"/> Elements</b></big> (grouped and sorted alphabetically):
				<ul style="margin-top:-1px;list-style-type:disc">
				<xsl:call-template name="output-element-index"/>
				</ul>
				<hr size="5"/>
				<br/>
				<xsl:apply-templates select="/dtd/externalSubset/contentModel">
					<xsl:sort select="@ename"/> 
				</xsl:apply-templates>
			</body>
		</html>
	</xsl:template>

	<xsl:template name="output-element-index">
		<xsl:param name="alphabet" select="$CONST_ALPHABET" />
		<xsl:if test="$alphabet">
			<xsl:call-template name="output-elements-by-letter">
				<xsl:with-param name="letter" select="substring($alphabet, 1, 1)" />
			</xsl:call-template>
			<xsl:call-template name="output-element-index">
				<xsl:with-param name="alphabet" select="substring($alphabet, 2)" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<xsl:template name="output-elements-by-letter">
		<xsl:param name="letter" select="'A'" />
		<xsl:variable name="element-set" select="//contentModel[upper-case(substring(@ename, 1, 1))=$letter]"/> <!-- starts-with(@ename,$letter) -->
		<xsl:if test="$element-set">
			<li><b><xsl:value-of select="$letter"/>: </b>
				<xsl:for-each select="$element-set">
					<xsl:sort select="@ename" />
					<xsl:variable name="cur_elem" select="@ename"/>
					<xsl:variable name="root" select="not(//element[@name=$cur_elem])"/>
					<xsl:choose>
						<xsl:when test="$root">
							<a href="#{@ename}"><b><xsl:value-of select="@ename"/></b></a>
						</xsl:when>
						<xsl:otherwise>
							<a href="#{@ename}"><xsl:value-of select="@ename"/></a>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:if test="position()!=last()">
						<xsl:text>, </xsl:text></xsl:if>
				</xsl:for-each>
			</li> </xsl:if>
	</xsl:template>


	<xsl:template match="contentModel">
		<xsl:variable name="cur_elem" select="@ename"/>
		<a name="{@ename}"/>
		<div style="border-color:#006699;border-width:2px;border-style:solid;">
			<table cellSpacing="2" cellPadding="4" border="0" width="100%">
				<tr>
					<td bgcolor="#eeeeee">
						<big>Element: <b><xsl:value-of select="@ename"/></b></big>
						<xsl:if test="($comments &gt; 1) and (name(./preceding-sibling::*[1])='comment')">
							<br/>
							<b>Description: </b>
							<xsl:apply-templates select="./preceding-sibling::comment[1]" mode="comment"/>
						</xsl:if>
					</td>
				</tr>
				<tr>
					<td>
						<table style="border-color:#006699;border-width:1px;border-style:solid;width:100%">
							<tr>
								<td bgcolor="#ccffff">
									<b>Content:</b>
									<xsl:choose>
										<xsl:when test="./empty">&#160;No children.</xsl:when>
										<xsl:when test="./any">&#160;Any children.</xsl:when>
										<xsl:otherwise>
											<ul style="margin-top:-2px; margin-bottom:0px">
												<xsl:apply-templates/>
											</ul>
										</xsl:otherwise>
									</xsl:choose>
								</td>
							</tr>
							<xsl:apply-templates select="./following-sibling::elementDecl[1]" mode="decl"/>
						</table>
					</td>
				</tr>
				<xsl:choose>
					<xsl:when test="//attlist[@ename=$cur_elem]">
						<tr>
							<td>
								<table style="border-color:#006699;border-width:1px;border-style:solid;width:100%">
									<tr>
										<td bgcolor="#ccffcc" align="center">                                            <!--	Attributes:  -->
											<table width="95%">
												<tr>
													<th align="left">Attribute</th>
													<th align="center">Type (Value)</th>
													<th align="center">Type (Definition)</th>
													<th align="center">Defaultvalue</th>
												</tr>
												<xsl:apply-templates select="//attributeDecl[@ename=$cur_elem][@fixed]" mode="attlist">
													<xsl:sort select="@aname"/>
												</xsl:apply-templates>
												<xsl:apply-templates select="//attributeDecl[@ename=$cur_elem][@required]" mode="attlist">
													<xsl:sort select="@aname"/>
												</xsl:apply-templates>
												<xsl:apply-templates select="//attributeDecl[@ename=$cur_elem][not(@required and @required)]" mode="attlist">
													<xsl:sort select="@aname"/>
												</xsl:apply-templates>
											</table>
										</td>
									</tr>
									<xsl:for-each select="//attlist[@ename=$cur_elem]">
										<xsl:if test="name(./preceding-sibling::*[1])='comment'">
											<tr>
												<td bgcolor="#eeeeee">Description of attribute
													<b><code>
														<xsl:value-of select="./attributeDecl/@aname"/>
													</code></b>:
													<xsl:apply-templates select="./preceding-sibling::comment[1]" mode="comment"/>
												</td>
											</tr>
										</xsl:if>
									</xsl:for-each>
								</table>
							</td>
						</tr>
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td bgcolor="#ccffcc">
								<i>No Attributes.</i>
							</td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				<tr>
					<td bgcolor="#ffffcc">
						<code><b><xsl:value-of select="@ename"/>&#160;</b></code>
						<xsl:choose>
							<xsl:when test="not(//element[@name=current()/@ename])">is never used
								<xsl:choose>
									<xsl:when test="$count_root_elements=1">(root element).</xsl:when>
									<xsl:when test="$count_root_elements&gt;1">(possible root element).</xsl:when>
								</xsl:choose>
 							</xsl:when>
							<xsl:otherwise>is&#160;used&#160;in:&#160;<xsl:for-each select="//contentModel[.//element[@name=$cur_elem]]">
									<xsl:sort select="@ename"/>
									<a href="#{@ename}"><xsl:value-of select="@ename"/></a>
									<xsl:if test="position() != last()">, </xsl:if>
								</xsl:for-each>.
							</xsl:otherwise>
						</xsl:choose>
					</td>
				</tr>
			</table>
		</div>
		<xsl:if test="position()!=last()">
			<p><a href="#top"><b>top</b></a></p>
		</xsl:if> 
	</xsl:template>

	<xsl:template match="attributeDecl" mode="attlist">
		<tr>
			<td>
				<xsl:choose>
					<xsl:when test="@required"><b><code><xsl:value-of select="@aname"/></code></b></xsl:when>
					<xsl:when test="@fixed"><code><xsl:value-of select="@aname"/></code></xsl:when>
					<xsl:otherwise><i><code><xsl:value-of select="@aname"/></code></i></xsl:otherwise>
				</xsl:choose>
			</td>
			<td align="center"><xsl:value-of select="replace(./@atype,'\|',' | ')"/></td>
			<td align="center">
				<xsl:choose>
					<xsl:when test="@required">required</xsl:when>
					<xsl:when test="@fixed">fixed</xsl:when>
					<xsl:otherwise>optional</xsl:otherwise>
				</xsl:choose>
			</td>
			<td align="center">
				<xsl:choose>
					<xsl:when test="@default">&quot;<xsl:value-of select="@default"/>&quot;</xsl:when>
					<xsl:otherwise>-</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
	</xsl:template>

	<xsl:template match="group">
		<li>
			<xsl:choose>
				<xsl:when test="./separator/@type = '|'">
					<xsl:choose>
						<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '*'"><i><code><xsl:value-of select="$SEPERATOR_PIPE_LABEL"/></code></i>&#160;<xsl:value-of select="$OCCURENCE_ASTERISK_LABEL"/></xsl:when>
						<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '+'"><code><xsl:value-of select="$SEPERATOR_PIPE_LABEL"/></code>&#160;<xsl:value-of select="$OCCURENCE_PLUS_LABEL"/></xsl:when>
						<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '?'"><i><code><xsl:value-of select="$SEPERATOR_PIPE_LABEL"/></code></i>&#160;<xsl:value-of select="$OCCURENCE_QUESTION_LABEL"/></xsl:when>
						<xsl:otherwise><code><xsl:value-of select="$SEPERATOR_PIPE_LABEL"/></code></xsl:otherwise>
					</xsl:choose>
					<ul style="margin-top:-2px; margin-bottom:0px; list-style-type:circle">
						<xsl:apply-templates>
							<xsl:sort select="@name"/>
						</xsl:apply-templates>
					</ul>
				</xsl:when>
				<xsl:otherwise> 

					<xsl:choose>
						<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '*'"><i><code><xsl:value-of select="$SEPERATOR_COMMA_LABEL"/></code></i>&#160;<xsl:value-of select="$OCCURENCE_ASTERISK_LABEL"/></xsl:when>
						<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '+'"><code><xsl:value-of select="$SEPERATOR_COMMA_LABEL"/></code>&#160;<xsl:value-of select="$OCCURENCE_PLUS_LABEL"/></xsl:when>
						<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '?'"><i><code><xsl:value-of select="$SEPERATOR_COMMA_LABEL"/></code></i>&#160;<xsl:value-of select="$OCCURENCE_QUESTION_LABEL"/></xsl:when>
						<xsl:otherwise><code><xsl:value-of select="$SEPERATOR_COMMA_LABEL"/></code></xsl:otherwise>
					</xsl:choose>
					<ul style="margin-top:-2px; margin-bottom:0px; list-style-type:square">
						<xsl:apply-templates/>
					</ul>
				</xsl:otherwise> 
			</xsl:choose>
		</li>
	</xsl:template>

	<xsl:template match="pcdata">
		<li>Text <code>(#PCDATA)</code></li>
	</xsl:template>

	<xsl:template match="element">
		<li>	
			<xsl:choose>
				<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '*'"><a href="#{@name}"><i><xsl:value-of select="@name"/></i></a>&#160;<xsl:value-of select="$OCCURENCE_ASTERISK_LABEL"/></xsl:when>
				<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '+'"><a href="#{@name}"><xsl:value-of select="@name"/></a>&#160;<xsl:value-of select="$OCCURENCE_PLUS_LABEL"/></xsl:when>
				<xsl:when test="./following-sibling::*[1][name()='occurrence']/@type = '?'"><a href="#{@name}"><i><xsl:value-of select="@name"/></i></a>&#160;<xsl:value-of select="$OCCURENCE_QUESTION_LABEL"/></xsl:when>
				<xsl:otherwise><a href="#{@name}"><xsl:value-of select="@name"/></a></xsl:otherwise>
			</xsl:choose></li>
	</xsl:template>

	<xsl:template match="elementDecl" mode="decl">
		<tr>
			<td bgcolor="#eeeeee">original&#160;declaration&#160;of&#160;children:&#160;<code><xsl:value-of select="replace(replace(./@model,'\|',' | '),',',', ')"/></code></td>
		</tr>
	</xsl:template>

	<xsl:template match="comment"></xsl:template>
	<xsl:template match="comment" mode="comment">
		<pre>
			<xsl:value-of select="."/>
		</pre>
	</xsl:template>
</xsl:stylesheet>